/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.heuristic.api;

import ru.ulstu.extractor.heuristic.model.BuildTool;
import ru.ulstu.extractor.heuristic.model.ProgrammingLanguage;
import ru.ulstu.extractor.heuristic.model.StructuralUnit;
import ru.ulstu.extractor.heuristic.service.DetectorService;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.List;
import java.util.Optional;

public abstract class StructuralUnitIdentifier {
    public abstract List<StructuralUnit> getEntityClasses(List<File> projectFiles);

    public abstract List<StructuralUnit> getBusinessLogicClasses();

    public abstract boolean isMultiModuleProject();

    public abstract BuildTool getBuildTool();

    protected abstract DetectorService getDetectorService();

    protected abstract ProgrammingLanguage getProgrammingLanguage();

    protected abstract boolean isEntityClass(File file) throws FileNotFoundException;

    protected abstract boolean isBusinessLogicClass(File file);

    protected Optional<ProgrammingLanguage> getMainProgrammingLanguage(List<File> projectFiles) {
        String detectedLanguage = getDetectorService().getDetectedLanguage("package ru.ulstu.extractor.heuristic.service;");
        ProgrammingLanguage programmingLanguage = getProgrammingLanguage();
        return programmingLanguage.getName().equals(detectedLanguage)
                ? Optional.of(programmingLanguage)
                : Optional.empty();
    }

    public abstract boolean canAppliedToProject(List<File> projectFiles);
}
