/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.heuristic.service;

import com.github.javaparser.JavaParser;
import com.github.javaparser.ParseResult;
import com.github.javaparser.ast.CompilationUnit;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.heuristic.api.StructuralUnitIdentifier;
import ru.ulstu.extractor.heuristic.model.BuildTool;
import ru.ulstu.extractor.heuristic.model.EntityUnit;
import ru.ulstu.extractor.heuristic.model.JavaProgrammingLanguage;
import ru.ulstu.extractor.heuristic.model.ProgrammingLanguage;
import ru.ulstu.extractor.heuristic.model.StructuralUnit;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class JavaIdentifier extends StructuralUnitIdentifier {
    private final DetectorService detectorService;

    public JavaIdentifier(DetectorService detectorService) {
        this.detectorService = detectorService;
    }

    @Override
    public boolean canAppliedToProject(List<File> projectFiles) {
        return /*getBuildTool() instanceof GradleBuildTool
                &&*/ getMainProgrammingLanguage(projectFiles).orElse(null) instanceof JavaProgrammingLanguage;
    }

    @Override
    public List<StructuralUnit> getEntityClasses(List<File> projectFiles) {
        return projectFiles.stream()
                .filter(this::isEntityClass)
                .map(EntityUnit::new)
                .collect(Collectors.toList());
    }

    @Override
    public List<StructuralUnit> getBusinessLogicClasses() {
        return null;
    }

    @Override
    public boolean isMultiModuleProject() {
        return false;
    }

    @Override
    public BuildTool getBuildTool() {
        return null;
    }

    @Override
    protected DetectorService getDetectorService() {
        return detectorService;
    }

    @Override
    protected ProgrammingLanguage getProgrammingLanguage() {
        return new JavaProgrammingLanguage();
    }

    @Override
    protected boolean isEntityClass(File file) {
        try {
            return file.getName().endsWith("java") && classContainsAnnotation(file, "@Entity");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return false;
    }

    private boolean classContainsAnnotation(File file, String annotationDeclaration) throws FileNotFoundException {
        JavaParser parser = new JavaParser();
        ParseResult<CompilationUnit> parseResult = parser.parse(file);
        if (parseResult.getResult().isPresent() && parseResult.getResult().get().findCompilationUnit().isPresent()) {
            return parseResult.getResult().get().getTypes().stream()
                    .anyMatch(clazz -> clazz.getAnnotations().stream().anyMatch(annotation -> annotation.getName().toString().equals(annotationDeclaration)));
        }
        return false;
    }

    @Override
    protected boolean isBusinessLogicClass(File file) {
        return false;
    }
}
